<!DOCTYPE html>
<html ng-app="myApp">

<head>
  <meta charset="utf-8">
  <title>Angular with jss</title>
  <script src="./app.js"></script>
</head>

<body ng-controller="myController">
  <button class="{{classes.button1}}">Button 1</button>
  <button class="{{classes.button2}}">Button 2</button>
</body>

</html>
