export default {
  bob: {
    '0%': {
      transform: 'translateY(0.4rem)'
    },
    '6.25%': {
      transform: 'translateY(-0.4rem)'
    },
    '12.5%': {
      transform: 'translateY(0.4rem)'
    },
    '18.75%': {
      transform: 'translateY(-0.4rem)'
    },
    '25%': {
      transform: 'translateY(0.4rem)'
    },
    '31.25%': {
      transform: 'translateY(-0.4rem)'
    },
    '37.5%': {
      transform: 'translateY(0.4rem)'
    },
    '43.75%': {
      transform: 'translateY(-0.4rem)'
    },
    '50%': {
      transform: 'translateY(0.4rem)'
    },
    '56.25%': {
      transform: 'translateY(-0.4rem)'
    },
    '62.5%': {
      transform: 'translateY(0.4rem)'
    },
    '68.75%': {
      transform: 'translateY(-0.4rem)'
    },
    '75%': {
      transform: 'translateY(0.4rem)'
    },
    '81.25%': {
      transform: 'translateY(-0.4rem)'
    },
    '87.5%': {
      transform: 'translateY(0.4rem)'
    },
    '93.75%': {
      transform: 'translateY(-0.4rem)'
    },
    '100%': {
      transform: 'translateY(0.4rem)'
    }
  },
  reverseSwing: {
    '0%': {
      transform: 'rotate(-5deg)'
    },
    '12.5%': {
      transform: 'rotate(10deg)'
    },
    '25%': {
      transform: 'rotate(-10deg)'
    },
    '37.5%': {
      transform: 'rotate(15deg)'
    },
    '50%': {
      transform: 'rotate(-23deg)'
    },
    '62.5%': {
      transform: 'rotate(23deg)'
    },
    '75%': {
      transform: 'rotate(-15deg)'
    },
    '87.5%': {
      transform: 'rotate(10deg)'
    },
    '100%': {
      transform: 'rotate(-5deg)'
    }
  },
  face: {
    '0%': {
      transform: 'translateX(-2.5px)'
    },
    '12.5%': {
      transform: 'translateX(5px)'
    },
    '25%': {
      transform: 'translateX(-5px)'
    },
    '37.5%': {
      transform: 'translateX(7.5px)'
    },
    '50%': {
      transform: 'translateX(-11.5px)'
    },
    '62.5%': {
      transform: 'translateX(11.5px)'
    },
    '75%': {
      transform: 'translateX(-7.5px)'
    },
    '87.5%': {
      transform: 'translateX(5px)'
    },
    '100%': {
      transform: 'translateX(-2.5px)'
    }
  },
  blink: {
    'from, to, 10%, 25%, 80%': {
      transform: 'scaleY(1)'
    },
    '8%, 23%, 78%': {
      transform: 'scaleY(0.1)'
    }
  },
  swingLeg: {
    '0%': {
      transform: 'rotate(0.5deg)'
    },
    '12.5%': {
      transform: 'rotate(-1deg)'
    },
    '25%': {
      transform: 'rotate(1deg)'
    },
    '37.5%': {
      transform: 'rotate(-1.5deg)'
    },
    '50%': {
      transform: 'rotate(2.3deg)'
    },
    '62.5%': {
      transform: 'rotate(-2.3deg)'
    },
    '75%': {
      transform: 'rotate(1.5deg)'
    },
    '87.5%': {
      transform: 'rotate(-1deg)'
    },
    '100%': {
      transform: 'rotate(0.5deg)'
    }
  },
  swingTail: {
    '0%': {
      transform: 'rotate(-2deg)'
    },
    '12.5%': {
      transform: 'rotate(4deg)'
    },
    '25%': {
      transform: 'rotate(-4deg)'
    },
    '37.5%': {
      transform: 'rotate(6deg)'
    },
    '50%': {
      transform: 'rotate(-9.2deg)'
    },
    '62.5%': {
      transform: 'rotate(9.2deg)'
    },
    '75%': {
      transform: 'rotate(-6deg)'
    },
    '87.5%': {
      transform: 'rotate(4deg)'
    },
    '100%': {
      transform: 'rotate(-2deg)'
    }
  },
  swing: {
    '0%': {
      transform: 'rotate(5deg)'
    },
    '12.5%': {
      transform: 'rotate(-10deg)'
    },
    '25%': {
      transform: 'rotate(10deg)'
    },
    '37.5%': {
      transform: 'rotate(-15deg)'
    },
    '50%': {
      transform: 'rotate(23deg)'
    },
    '62.5%': {
      transform: 'rotate(-23deg)'
    },
    '75%': {
      transform: 'rotate(15deg)'
    },
    '87.5%': {
      transform: 'rotate(-10deg)'
    },
    '100%': {
      transform: 'rotate(5deg)'
    }
  }
}
