// react-jss, css-jss
import 'core-js/features/map'
import 'core-js/features/function/name'
// jss, jss-plugin-cache
import 'core-js/features/weak-map'
// styled-system
import 'core-js/features/set'
import 'core-js/features/symbol'
// react-jss, css-jss
import 'core-js/features/object/assign'
// tests
import 'core-js/features/typed-array'
import 'core-js/features/string/includes'
// examples
import 'raf/polyfill'
