export const colorBg = '#1F1F3C'
export const colorFurLight = '#D5E8F8'
export const colorFurDark = '#8B9BD9'
export const colorFeatures = '#4B4D75'
export const colorFeaturesLight = '#9FA2CB'
export const colorYarn = '#DB242A'
export const easing = 'cubic-bezier(0.5, 0, 0.5, 1)'
export const offset = '.5s'
export const duration = 5
